#!/usr/bin/env node
const download = require("download-git-repo"); // git下载
const program = require("commander");
const exists = require("fs").existsSync; // exists(path):检测路径是否存在，返回boolean
const ora = require("ora"); // loading效果
const chalk = require("chalk"); // 添加颜色
const rm = require("rimraf").sync; // unix命令 删除文件
const templatename = require("../template");
const generate = require("../lib/generate");
/**
 * 创建模板.
 */
/**
 * 用法
 * template-name 模板名称  vue  react   applet
 * project-name  项目名称
 */

program.usage("<template-name> [project-name]");

/**
 * 帮助
 */

program.on("--help", () => {
  console.log("  例子:");
  console.log(
    chalk.gray("    $ hx init 模板名称(vue  react   applet) 项目名称")
  );
});

function help() {
  program.parse(process.argv);
  if (!program.args.length) return program.help();
}
help();

/**
 * 设置
 */
let templateName = program.args[0]; // 获取模板名称
const tmp = templatename.find((item) => item.name === templateName);
console.log(templateName, "program.args");
const rawName = program.args[1]; // 获取项目名称
IsURL(templateName, rawName, tmp);

/**
 * 出口
 */
console.log();
process.on("exit", () => {
  console.log();
});
function IsURL(strUrl, rawName, tmp) {
  var regular =
    /^\b(((https?|ftp):\/\/)?[-a-z0-9]+(\.[-a-z0-9]+)*\.(?:com|edu|gov|int|mil|net|org|biz|info|name|museum|asia|coop|aero|[a-z][a-z]|((25[0-5])|(2[0-4]\d)|(1\d\d)|([1-9]\d)|\d))\b(\/[-a-z0-9_:\@&?=+,.!\/~%\$]*)?)$/i;
  if (regular.test(strUrl)) {
    downloadAndGenerate(rawName, strUrl);
    return true;
  } else {
    intRun(tmp, rawName);
    return false;
  }
}

/**
 * 检查、模板是否存在 项目名称是否为空
 * @param {*} templateName
 * @param {*} rawName
 */
function intRun(templateName, projectName) {
  generate(projectName, templateName, (err) => {
    downloadAndGenerate(projectName, templateName);
  });
}
/**
 * 从git中拉项目
 * @param {string} projectName
 * @param {string} templateUrl
 */
function downloadAndGenerate(projectName, templateUrl) {
  const spinner = ora("下载模板");
  spinner.start("开始下载模板");
  // 删除本地模板是否存在
  if (exists(projectName)) rm(projectName);
  download(`direct:${templateUrl}`, projectName, { clone: true }, (err) => {
    spinner.stop();
    if (err) return console.log(chalk.red(`模板生成失败：${err}`));
    spinner.succeed("模板下载成功");
  });
}
